{% extends "base.html" %}

{% block title %}Contact {{ site.title }}{% endblock %}

{% block main %}
    <h1>Contact {{ site.title }}</h1>

    <form action="https://formspree.io/{{ site.email }}" method="POST">
        <div class="row control-group">
            <div class="form-group col-xs-12 floating-label-form-group controls">
                <label>Your email</label>
                <input ref="emailInput" type="email" class="form-control" placeholder="Your email" name="_replyto">
            </div>
        </div>
        <div class="row control-group">
            <div class="form-group col-xs-12 floating-label-form-group controls">
                <label>Message</label>
                <textarea ref="messageInput" rows="8" class="form-control" placeholder="Message" name="message"></textarea>
            </div>
        </div>

        <input type="hidden" name="_subject" value="New message on {{ site.title }}" />
        <input type="text" name="_gotcha" style="display:none" />

        <br>

        <button type="submit" class="btn btn-primary btn-lg"><i class="glyphicon glyphicon-envelope"></i>&nbsp;&nbsp;Send</button>
    </form>
{% endblock main %}

