<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta content="IE=edge" http-equiv="X-UA-Compatible">
    <meta content="width=device-width, initial-scale=1" name="viewport">
    <meta name="generator" content="VictorPy {{ site.engine }}" />

    <title>{% block title %}{{ title }}{% endblock title %}</title>

    {% if description %}
        <meta description="{{ description }}">
    {% endif %}

    {% if keywords %}
        <meta keywords="{% if keywords %}{{ keywords|join(', ') }}{% endif %}">
    {% endif %}

    <link href="https://cdnjs.cloudflare.com/ajax/libs/bootswatch/3.3.7/{{ site.theme if site.theme else "flatly" }}/bootstrap.min.css" rel="stylesheet">

    <link rel="stylesheet" href="/static/css/borland.css">

    <style>
        body {
            padding-top: 80px;
            font-size: 18px;
        }
    </style>

    <link rel="alternate" type="application/rss+xml" title="RSS" href="{{ site.base_url }}/index.xml">
    {% block before_endhead %}{% endblock before_endhead %}
</head>
<body>
    <nav class="navbar navbar-inverse navbar-fixed-top">
        <div class="container">
            <div class="navbar-header">
                <button aria-controls="navbar" aria-expanded="false" class="navbar-toggle collapsed" data-target="#navbar" data-toggle="collapse" type="button"><span class="sr-only">Toggle navigation</span> <span class="icon-bar"></span> <span class="icon-bar"></span> <span class="icon-bar"></span></button> <a class="navbar-brand" href="/">{{ site.title }}</a>
            </div>
            <div class="collapse navbar-collapse" id="navbar">
                <ul class="nav navbar-nav">
                     {% for section in site.root_sections %}
                        <li{% if section.permalink == permalink %} class="active"{% endif %}>
                            <a href="{{ section.url }}">{% if section.menuTitle %}{{ section.menuTitle }}{% else %}{{ section.title }}{% endif %}</a>
                        </li>
                    {% endfor %}
                </ul>
            </div>
        </div>
    </nav>

    <div class="container">
        {% block main %}{% endblock main %}
    </div>


    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>
    <script src="https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/3.3.7/js/bootstrap.min.js"></script>

    <script>
        $("table").addClass("table table-hover");
        $("div.page img").addClass("img-responsive");
    </script>

    {% block before_endbody %}{% endblock before_endbody %}

</body>
</html>