{% extends "base.html" %}

{% block title %}{{ title|title }}{% endblock title %}

{% block main %}

    <h1>{{ title|title }}</h1>

    <ul>
        # for p in site.tags.get(title).pages
            <li><a href="{{ p.url }}">{{ p.title }}</a></li>
        # endfor
    </ul>

{% endblock main %}