{% extends "base.html" %}

{% block title %}{{ title }}{% endblock title %}

{% block main %}

    <h1>Tags</h1>

    <ul>
        # for tag, p in site.tags.items()
            <li><a href="{{ p.url }}">{{ p.title }}</a></li>
        # endfor
    </ul>

{% endblock main %}
