<div id="disqus_thread"></div>

<script>
var disqus_config = function () {
this.page.url = {{ permalink }};
this.page.identifier = {{ unique_id }};
};

(function() {
var d = document, s = d.createElement('script');
s.src = '//{{ site.disqus_id }}.disqus.com/embed.js';
s.setAttribute('data-timestamp', +new Date());
(d.head || d.body).appendChild(s);
})();
</script>
<noscript>Please enable JavaScript to view the <a href="https://disqus.com/?ref_noscript">comments powered by Disqus.</a></noscript>
